// db.js
// import Dexie from 'dexie';

class PromptDatabase extends Dexie {
    constructor() {
        super('PromptDatabase'); // Set the database name

        // Define database schema
        this.version(1).stores({
            prompts: '++id, &name, prompt, category, tag, user, others',
            // prots: '++id, &name, prompt, category, tag, user, others',
            prots: '++id, &name, post_id, imagelink,contents',
            setting: '++id, &name, content, category',
            promptpart: '++id, &name, ahead, content, category',
            jobslog: '++id, product_by_who, post_id, post_title, post_time'
        });

        // Create a reference to the table
        this.prompts = this.table('prompts');
        this.setting = this.table('setting');
        this.promptpart = this.table('promptpart'); // Reference to promptpart table
        this.jobslog = this.table('jobslog'); // Reference to jobslog table

        this.promptcate = this.getAllCategory();
        this.promptag = this.getAllTag();

    }

     /**
     * Add a job log entry
     * @param {Object} job - Job log entry
     * @param {string} job.product_by_who - Who created the product
     * @param {number} job.post_id - ID of the post
     * @param {string} job.post_title - Title of the post
     * @param {string} job.post_time - Timestamp of the post
     * @returns {Promise<number>} - ID of the added entry
     */
    async addJobLog(job) {
        return await this.jobslog.add(job);
    }

    /**
     * Delete a job log entry by ID
     * @param {number} id - ID of the job log to delete
     * @returns {Promise<void>}
     */
    async deleteJobLog(id) {
        return await this.jobslog.delete(id);
    }

    /**
     * Get all job log entries
     * @returns {Promise<Array>} - Array of job log entries
     */
    async getAllJobLogs() {
        return await this.jobslog.toArray();
    }

    /**
     * Clear all job log entries
     * @returns {Promise<void>}
     */
    async clearJobLogs() {
        return await this.jobslog.clear();
    }

    // Example Usage
// async function exampleUsage() {
//     const db = new PromptDatabase();

//     // Adding a job log
//     const jobId = await db.addJobLog({
//         product_by_who: "Alice",
//         post_id: 101,
//         post_title: "New Feature Release",
//         post_time: new Date().toISOString()
//     });
//     console.log("Added job log with ID:", jobId);

//     // Retrieving all job logs
//     const jobLogs = await db.getAllJobLogs();
//     console.log("All Job Logs:", jobLogs);

//     // Deleting a job log
//     await db.deleteJobLog(jobId);
//     console.log("Deleted job log with ID:", jobId);

//     // Clearing all job logs
//     await db.clearJobLogs();
//     console.log("Cleared all job logs.");
// }

    async deleteByCategory(category) {
        try {
            const deletedCount = await this.prompts
                .where('category')
                .equals(category) // Match category
                .delete(); // Delete all matched records

            console.log(`${deletedCount} records with category "${category}" deleted.`);
        } catch (error) {
            console.error('Error deleting records by category:', error);
        }
    }

    // Method to delete records by tag
    async deleteByTag(tag) {
        try {
            const deletedCount = await this.prompts
                .where('tag')
                .equals(tag) // Match tag
                .delete(); // Delete all matched records

            console.log(`${deletedCount} records with tag "${tag}" deleted.`);
        } catch (error) {
            console.error('Error deleting records by tag:', error);
        }
    }

    //操作promptpart表
    // Method to read all entries in the 'promptpart' table
    // db.getAllPromptParts().then(data => console.log(data));
    // db.getAllPromptParts().then(data => console.log(data));
    // db.getPromptPartByName('Example').then(data => console.log(data));
    // db.deletePromptPartByName('Example').then(() => console.log('Deleted entry'));

    async getAllPromptParts() {
        try {
            return await this.promptpart.toArray();
        } catch (error) {
            console.error('Error reading prompt parts:', error);
        }
    }

    // Method to read a specific entry by name
    async getPromptPartByName(name) {
        try {
            return await this.promptpart.where('name').equals(name).first();
        } catch (error) {
            console.error('Error reading prompt part by name:', error);
        }
    }

    // Method to add or update an entry by name
    async addOrUpdatePromptPartByName(data) {
        //example
        // db.addOrUpdatePromptPartByName({
        //     name: 'Example', // 使用 name 作为唯一标识
        //     ahead: 'Important info',
        //     content: 'This is a test prompt part',
        //     category: 'This is a test , category',
        // });
        try {
            const existing = await this.getPromptPartByName(data.name);
            if (existing) {
                // Update if name exists
                await this.promptpart.update(existing.id, data);
                console.log('Updated prompt part:', data);
            } else {
                // Add if name does not exist
                await this.promptpart.add(data);
                console.log('Added new prompt part:', data);
            }
        } catch (error) {
            console.error('Error adding or updating prompt part:', error);
        }
    }

    // Method to delete an entry by name
    async deletePromptPartByName(name) {
        try {
            const existing = await this.getPromptPartByName(name);
            if (existing) {
                await this.promptpart.delete(existing.id);
                console.log(`Deleted prompt part with name: ${name}`);
            } else {
                console.warn(`No prompt part found with name: ${name}`);
            }
        } catch (error) {
            console.error('Error deleting prompt part by name:', error);
        }
    }

    // 添加新方法：获取属于某个 category 的所有记录
    async getPromptPartsByCategory(category) {
        try {
            const results = await this.promptpart.where('category').equals(category).toArray();
            console.log(`Records in category "${category}":`, results);
            return results;
        } catch (error) {
            console.error('Error fetching records by category:', error);
            return [];
        }
    }
   //操作promptpart表
    
    // 操作prompts表
    // Method to check if prompts have less than 10 records


    async checkPromptCount() {
        try {
            const count = await this.prompts.count(); // Get total number of records
            return count < 10 ? 0 : count;
        } catch (error) {
            console.error('Error checking prompt count:', error);
            return 0; // Return 0 if there is any error
        }
    }

    
    // Get tags by category
    async getTagsByCategory(categoryValue) {
        try {
            // 获取所有属于该 category 的 prompt 记录
            const promptsInCategory = await this.prompts.where('category').equals(categoryValue).toArray();

            // 提取所有的 tag 字段
            const tags = promptsInCategory.map(item => item.tag);

            // 去重
            const uniqueTags = [...new Set(tags)];

            // 返回去重后的标签列表
            return uniqueTags;
        } catch (error) {
            console.error('Failed to get tags by category:', error);
        }
    }

    // Add a new prompt
    async addPrompt({ name, prompt, category, tag, user, others }) {
        // {
        //     name: 'promptValue',
        //     prompt: 'arr',
        //     category: 'categoryValue',
        //     tag: 'tagValue',
        //     user: 'user',
        //     others: 'others'
        // }
        // 查找是否已有相同 name 的记录
        try {
            // throw new UserException('InvalidMonthNo');
            const existingPrompt = await this.prompts.where('name').equals(name).first();
            // console.log(existingPrompt)
            if (existingPrompt) {
                // 如果存在，则更新数据
                await this.prompts.update(existingPrompt.id, { prompt, category, tag, user, others });
                console.log(`Prompt with name "${name}" updated.`);
                return "updated"
            } else {
                // 如果不存在，则添加新数据
                await this.prompts.add({ name, prompt, category, tag, user, others });
                console.log(`Prompt with name "${name}" added.`);
                return "added"
            }
        } catch (error) {
            return "failed"
        }
    }

    // Get all prompts
    async getAllPrompts() {
        try {
            return await this.prompts.toArray();
        } catch (error) {
            console.error('Failed to fetch prompts:', error);
        }
    }

    async getPromptByName(name) { 
       const lowerName = name.toLowerCase();
        return await this.prompts.filter(item => item.name.toLowerCase() === lowerName).first();
    }
    async searchPrompt(input) { 
      const lowerInput = input.toLowerCase();
        return await this.prompts.filter(item => item.name.toLowerCase().includes(lowerInput)).toArray();
    }
    async searchPromptag(input) { 
        const lowerInput = input.toLowerCase();
        return await this.prompts.filter(item => item.tag.toLowerCase().includes(lowerInput)).toArray();
    }
    async searchPromptcate(input) { 
        const lowerInput = input.toLowerCase();
        return await this.prompts.filter(item => item.category.toLowerCase().includes(lowerInput)).toArray();
    }

    // Update prompt by name
    async updatePromptByName(name, updates) {
        try {
            const prompt = await this.prompts.where('name').equals(name).first();
            if (prompt) {
                await this.prompts.update(prompt.id, updates);
                console.log(`Prompt with name "${name}" updated.`);
            } else {
                console.log(`Prompt with name "${name}" not found.`);
            }
        } catch (error) {
            console.error('Failed to update prompt by name:', error);
        }
    }

    // Delete prompt by name
    async deletePromptByName(name) {
        try {
            const prompt = await this.prompts.where('name').equals(name).first();
            if (prompt) {
                await this.prompts.delete(prompt.id);
                console.log(`Prompt with name "${name}" deleted.`);
            } else {
                console.log(`Prompt with name "${name}" not found.`);
            }
        } catch (error) {
            console.error('Failed to delete prompt by name:', error);
        }
    }

    async getAllPromptByCategory(category) {
        const lowerCategory = category.toLowerCase();
        return await this.prompts.filter(item => item.category.toLowerCase() === lowerCategory).toArray();
    }
    async getAllPromptByTag(tag) {
        const lowerTag = tag.toLowerCase();
        return await this.prompts.filter(item => item.tag.toLowerCase() === lowerTag).toArray();
    }

    async getAllCategory() {
        const categories = await this.prompts.toArray();
        const categoryList = categories.map(item => item.category);
        // 去重
        const uniqueCategories = [...new Set(categoryList)];
        // console.log("Unique Categories:", uniqueCategories);
        return uniqueCategories;
    }

    async getAllTag() {
        const tags = await this.prompts.toArray();
        const tagsList = tags.map(item => item.tag);
        // 去重
        const uniqueTags = [...new Set(tagsList)];
        // console.log("Unique Categories:", uniqueTags);
        return uniqueTags;
    }
    async getTagsByName(input) {
        const lowerInput = input.toLowerCase();
        const tags = await this.prompts.filter(item => item.tag.toLowerCase().startsWith(lowerInput)).toArray();
        const tagsList = tags.map(item => item.tag);
        // 去重
        const uniqueTags = [...new Set(tagsList)];
        // console.log("Unique Categories:", uniqueTags);
        return uniqueTags;
    }

    async getRecordsByTag(tagValue) {
        const lowerTag = tagValue.toLowerCase();
        return await this.prompts.filter(item => item.tag.toLowerCase() === lowerTag).toArray();
    }

    async addSetting({ name, content, category }) {
        // db.addSetting({
        //     name: 'seokwResearch',
        //     content: ``,
        //     category: 'SEO',
        // })
        // 查找是否已有相同 name 的记录
        try {
            // throw new UserException('InvalidMonthNo');
            const existing = await this.setting.where('name').equals(name).first();
            // console.log(existingPrompt)
            if (existing) {
                // 如果存在，则更新数据
                await this.setting.update(existing.id, { name, content, category });
                console.log(`Prompt with name "${name}" updated.`);
                return "updated"
            } else {
                // 如果不存在，则添加新数据
                await this.setting.add({ name, content, category });
                console.log(`Prompt with name "${name}" added.`);
                return "added"
            }
        } catch (error) {
            return "failed"
        }
    }
}

// export default PromptDatabase;
