class LocalStorageCache {
    constructor(key, maxLength = 10) {
        this.key = key; // 缓存在 localStorage 的键名
        this.maxLength = maxLength; // 最大缓存长度
        this.load(); // 初始化加载缓存
    }

    // 加载缓存数据
    load() {
        const data = localStorage.getItem(this.key);
        this.cache = data ? JSON.parse(data) : [];
    }

    // 保存缓存数据到 localStorage
    save() {
        localStorage.setItem(this.key, JSON.stringify(this.cache));
    }

    // 添加数据到最前面
    add(item) {
        if (typeof item !== 'object') {
            throw new Error('Only JSON objects can be added to the cache.');
        }

        // 检查 item 是否已存在（根据深度值匹配，或根据特定的字段）
        const exists = this.cache.some(cachedItem => JSON.stringify(cachedItem) === JSON.stringify(item));

        if (!exists) {
            // 将新数据添加到最前面
            this.cache.unshift(item);

            // 如果超过最大长度，移除最旧数据（数组末尾的元素）
            if (this.cache.length > this.maxLength) {
                this.cache.pop();
            }

            // 保存缓存数据
            this.save();
            return true
        } else {
            return false
        }
    }


    // 获取所有数据（按时间排序，最新在前）
    getAll() {
        return [...this.cache]; // 返回数据副本，避免直接修改
    }

    // 查找数据（支持自定义筛选函数）
    find(predicate) {
        return this.cache.find(predicate);
    }

    // 删除数据（支持自定义筛选函数）
    remove(predicate) {
        this.cache = this.cache.filter(item => !predicate(item));
        this.save();
    }

    // 清空缓存
    clear() {
        this.cache = [];
        this.save();
    }

    // 更新数据（根据筛选条件找到并更新）
    update(predicate, newItem) {
        const index = this.cache.findIndex(predicate);
        if (index !== -1) {
            this.cache[index] = { ...this.cache[index], ...newItem };
            this.save();
        }
    }
}

// // 使用示例
// const myCache = new LocalStorageCache('my_cache', 10); // 缓存数组最大长度为 10

// // 添加数据
// myCache.add({ id: 1, name: 'Item 1', value: 100 });
// myCache.add({ id: 2, name: 'Item 2', value: 200 });
// myCache.add({ id: 3, name: 'Item 3', value: 300 });

// // 获取所有数据
// console.log('All data:', myCache.getAll());

// // 查找数据
// const found = myCache.find(item => item.id === 2);
// console.log('Found item:', found);

// // 更新数据
// myCache.update(item => item.id === 2, { value: 250 });
// console.log('Updated data:', myCache.getAll());

// // 删除数据
// myCache.remove(item => item.id === 3);
// console.log('After deletion:', myCache.getAll());

// // 清空缓存
// myCache.clear();
// console.log('After clearing:', myCache.getAll());
